// agl_nano_diag_r1_top.v
///////////////////////////////////////////////////////////////////////////////////////////////////
// Company: System Level Solutions, Inc.
//
// File: agl_nano_diag_r1_top.v
// File history:
//      Rev. 1.0.0: 14-Oct-2008: Initial Design
//      Rev. 1.0.1: 5/21/09: add syn_global_buffers = 2 attribute to module 
//
// Description: 
//
// This design is used to perform diagnostic test of IGLOO Nano Starter Kit
//
// Targeted device: IGLOO AGL250V2 100VQFP
// Author: Ketan Vaghela
//
/////////////////////////////////////////////////////////////////////////////////////////////////// 
module agl_nano_diag_r1_top ( AGL_clk, AGL_reset_n, AGL_ff, AGL_rxd, AGL_txd, LED, DIP_SW, PB_SW, GPIO)/* synthesis syn_global_buffers = 2 */;
  input          AGL_clk;
  input          AGL_reset_n;
  input          AGL_ff;

  input          AGL_rxd;
  output         AGL_txd;

  output [07:00] LED;
  input  [07:00] DIP_SW;
  input  [03:00] PB_SW;

  output [42:00] GPIO;  

  wire           int_clk;    
  wire           ff_enabled;
  wire   [07:00] uart_data;
  wire           tx_ready;
  wire           rx_ready;
  wire           uart_overflow;
  wire           parity_error;
  wire           uart_we_n = ~(rx_ready & tx_ready & ~uart_overflow);
  wire           uart_oe_n = uart_we_n;
  wire           pll_lock;
  wire           int_reset_n;
  wire           sys_reset_n = int_reset_n /*& pll_lock*/;
  wire           sys_clk;
  wire   [42:00] GPIO = {43{sys_clk}};

  reg    [03:00] pb_status;
  reg    [18:00] counter;
  reg    [07:00] sh_rgt;
  reg    [07:00] sh_lft;
  reg    [07:00] LED;
  
  
CLKINT reset_n (.A (AGL_reset_n), .Y(int_reset_n));


//PLL
pll_750khz static_pll (
        .POWERDOWN ( int_reset_n ),
        .CLKA      ( AGL_clk     ),
        .LOCK      ( pll_lock    ),
        .GLA       ( sys_clk     ) );


// Flash-Freeze 
agl_nano_ff flash_freeze (
		.Flash_Freeze_N       ( AGL_ff            ),
		.RST_N                ( int_reset_n       ),
		.CLK                  ( AGL_clk           ),
        .CLK_GATED            ( int_clk           ),        
        .Flash_Freeze_Enabled ( ff_enabled        ) );

  
//UART
agl_nano_uart uart (
        .BAUD_VAL   ( 8'h4          ),
        .BIT8       ( 1'b1          ),
        .CLK        ( sys_clk       ),
        .CSN        ( ff_enabled    ),
        .DATA_IN    ( uart_data     ),
        .DATA_OUT   ( uart_data     ),
        .ODD_N_EVEN ( 1'b0          ),
        .OEN        ( uart_oe_n     ),
        .OVERFLOW   ( uart_overflow ),
        .PARITY_EN  ( 1'b0          ),
        .PARITY_ERR ( parity_error  ),
        .RESET_N    ( sys_reset_n   ),
        .RX         ( AGL_rxd       ),
        .RXRDY      ( rx_ready      ),
        .TX         ( AGL_txd       ),
        .TXRDY      ( tx_ready      ),
        .WEN        ( uart_we_n     ) );

//PushButton 
  always @ (posedge sys_clk or negedge sys_reset_n)
    begin
      if (~sys_reset_n)
        pb_status <= 4'b0000;      
      else
        casex (PB_SW)
          4'bxxx0  : pb_status <= 4'b0001;
          4'bxx01  : pb_status <= 4'b0010;
          4'bx011  : pb_status <= 4'b0100;
          4'b0111  : pb_status <= 4'b1000;
          default  : pb_status <= pb_status;
        endcase
    end

//free running binary counter
  always @ (posedge sys_clk or negedge sys_reset_n)
    begin
      if (~sys_reset_n)
         counter <= 19'b0;      
      else
         counter <= counter + 19'b1;
    end

//shift right
  always @ (posedge counter[17] or negedge sys_reset_n)
    begin
      if (~sys_reset_n)
        sh_rgt <= 8'hFE;
      else
        sh_rgt <= {sh_rgt[6:0], sh_rgt[7]};
    end

//shift left
  always @ (posedge counter[17] or negedge sys_reset_n)
    begin
      if (~sys_reset_n)
        sh_lft <= 8'h7F;
      else
        sh_lft <= {sh_lft[0], sh_lft[7:1]};  
    end  

//LED display
  always @ (*)
    begin
      if (~sys_reset_n)
        LED <= 8'hFF;
      else
        casex (pb_status)
          4'b0001  : LED <= {8{counter[18]}};
          4'b0010  : LED <= sh_rgt;
          4'b0100  : LED <= sh_lft;
          4'b1000  : LED <= DIP_SW;
          default  : LED <= 8'hFF;
        endcase
    end

endmodule
